@@ -39,6 +39,7 @@
 #include <string.h>
 #include <webkit2/webkit-web-extension.h>
 #define WEBKIT_DOM_USE_UNSTABLE_API
+#include <webkitdom/WebKitDOMCustomUnstable.h>
 #include <webkitdom/WebKitDOMDOMWindowUnstable.h>
 #include <JavaScriptCore/JavaScript.h>
 
@@ -111,11 +112,13 @@ web_page_send_request (WebKitWebPage     *web_page,
                        WebKitURIResponse *redirected_response,
                        EphyWebExtension  *extension)
 {
+  const char *original_request_uri;
   const char *request_uri;
   const char *page_uri;
   gboolean ret;
 
   request_uri = webkit_uri_request_get_uri (request);
+  original_request_uri = request_uri;
 
   if (g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_DO_NOT_TRACK)) {
     SoupMessageHeaders *headers;
@@ -143,7 +146,7 @@ web_page_send_request (WebKitWebPage     *web_page,
   page_uri = webkit_web_page_get_uri (web_page);
 
   /* Always load the main resource. */
-  if (g_strcmp0 (request_uri, page_uri) == 0)
+  if (g_strcmp0 (original_request_uri, page_uri) == 0)
     return FALSE;
 
   /* Always load data requests, as uri_tester won't do any good here. */
@@ -362,20 +365,26 @@ fill_form_cb (const char *username,
               gpointer    user_data)
 {
   EphyEmbedFormAuth *form_auth = EPHY_EMBED_FORM_AUTH (user_data);
-  WebKitDOMNode *username_node;
+  WebKitDOMHTMLInputElement *username_node;
+  WebKitDOMHTMLInputElement *password_node;
 
   if (username == NULL && password == NULL) {
     LOG ("No result");
     return;
   }
 
-  username_node = ephy_embed_form_auth_get_username_node (form_auth);
+  username_node = WEBKIT_DOM_HTML_INPUT_ELEMENT (ephy_embed_form_auth_get_username_node (form_auth));
+  password_node = WEBKIT_DOM_HTML_INPUT_ELEMENT (ephy_embed_form_auth_get_password_node (form_auth));
 
   LOG ("Found: user %s pass (hidden)", username_node ? username : "(none)");
-  if (username_node)
-    g_object_set (username_node, "value", username, NULL);
-  g_object_set (ephy_embed_form_auth_get_password_node (form_auth),
-                "value", password, NULL);
+  if (username_node) {
+    g_object_set_data (G_OBJECT (username_node), "ephy-is-auto-filling", GINT_TO_POINTER (TRUE));
+    webkit_dom_html_input_element_set_auto_filled (username_node, TRUE);
+    webkit_dom_html_input_element_set_editing_value (username_node, username);
+    g_object_set_data (G_OBJECT (username_node), "ephy-is-auto-filling", GINT_TO_POINTER (FALSE));
+  }
+  webkit_dom_html_input_element_set_auto_filled (password_node, TRUE);
+  webkit_dom_html_input_element_set_editing_value (password_node, password);
 }
 
 static gint
@@ -882,6 +891,9 @@ username_node_input_cb (WebKitDOMNode  *username_node,
   WebKitDOMDocument *document;
   WebKitDOMElement *main_div;
 
+  if (g_object_get_data (G_OBJECT (username_node), "ephy-is-auto-filling"))
+    return TRUE;
+
   g_object_set_data (G_OBJECT (username_node), "ephy-user-ever-edited", GINT_TO_POINTER (TRUE));
   document = webkit_web_page_get_dom_document (web_page);
   remove_user_choices (document);
@@ -1176,12 +1188,12 @@ handle_method_call (GDBusConnection       *connection,
   } else if (g_strcmp0 (method_name, "GetBestWebAppIcon") == 0) {
     WebKitWebPage *web_page;
     WebKitDOMDocument *document;
-    char *base_uri = NULL;
+    const char *base_uri = NULL;
     char *uri = NULL;
     char *color = NULL;
     guint64 page_id;
 
-    g_variant_get (parameters, "(ts)", &page_id, &base_uri);
+    g_variant_get (parameters, "(t&s)", &page_id, &base_uri);
     web_page = get_webkit_web_page_or_return_dbus_error (invocation, extension->extension, page_id);
     if (!web_page)
       return;
@@ -1284,7 +1296,9 @@ ephy_web_extension_dispose (GObject *object)
 {
   EphyWebExtension *extension = EPHY_WEB_EXTENSION (object);
 
-  g_clear_object (&extension->uri_tester);
+// Leaked in gnome-3-22 to avoid crashes when destroyed in the exit handler.
+// Fixed in master by moving the UriTester to the UI process.
+//  g_clear_object (&extension->uri_tester);
   g_clear_object (&extension->overview_model);
   g_clear_pointer (&extension->form_auth_data_cache,
                    ephy_form_auth_data_cache_free);
@@ -1299,7 +1313,10 @@ ephy_web_extension_dispose (GObject *object)
     extension->page_created_signals_pending = NULL;
   }
 
-  g_clear_object (&extension->cancellable);
+  if (extension->cancellable) {
+    g_cancellable_cancel (extension->cancellable);
+    g_clear_object (&extension->cancellable);
+  }
   g_clear_object (&extension->dbus_connection);
 
   g_clear_object (&extension->extension);
@@ -1351,6 +1368,7 @@ dbus_connection_created_cb (GObject          *source_object,
   if (error) {
     g_warning ("Failed to connect to UI process: %s", error->message);
     g_error_free (error);
+    g_object_unref (extension);
     return;
   }
 
@@ -1366,11 +1384,14 @@ dbus_connection_created_cb (GObject          *source_object,
     g_warning ("Failed to register web extension object: %s\n", error->message);
     g_error_free (error);
     g_object_unref (connection);
+    g_object_unref (extension);
     return;
   }
 
   extension->dbus_connection = connection;
   ephy_web_extension_emit_page_created_signals_pending (extension);
+
+  g_object_unref (extension);
 }
 
 static gboolean
@@ -1418,6 +1439,6 @@ ephy_web_extension_initialize (EphyWebExtension   *extension,
                                      observer,
                                      extension->cancellable,
                                      (GAsyncReadyCallback)dbus_connection_created_cb,
-                                     extension);
+                                     g_object_ref (extension));
   g_object_unref (observer);
 }
