@@ -39,6 +39,13 @@ struct _EphySnapshotService {
 
 G_DEFINE_TYPE (EphySnapshotService, ephy_snapshot_service, G_TYPE_OBJECT)
 
+enum {
+  SNAPSHOT_SAVED,
+  LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL];
+
 typedef enum {
   SNAPSHOT_STALE,
   SNAPSHOT_FRESH
@@ -59,6 +66,24 @@ snapshot_path_cached_data_free (SnapshotPathCachedData *data)
 static void
 ephy_snapshot_service_class_init (EphySnapshotServiceClass *klass)
 {
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+
+  /**
+   * EphySnapshotService::snapshot-saved:
+   * @url: the URL the snapshot was saved for
+   * @mtime: the mtime embedded in the snapshot, needed to retrieve it
+   *
+   * The ::snapshot-saved signal is emitted when a new snapshot is saved.
+   **/
+  signals[SNAPSHOT_SAVED] = g_signal_new ("snapshot-saved",
+                                          G_OBJECT_CLASS_TYPE (object_class),
+                                          G_SIGNAL_RUN_LAST,
+                                          0,
+                                          NULL, NULL, NULL,
+                                          G_TYPE_NONE,
+                                          2,
+                                          G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
+                                          G_TYPE_INT64);
 }
 
 static void
@@ -133,66 +158,168 @@ ephy_snapshot_service_prepare_snapshot (cairo_surface_t *surface,
 }
 
 typedef struct {
+  EphySnapshotService *service;
+  GdkPixbuf *snapshot;
   WebKitWebView *web_view;
   time_t mtime;
-  gboolean for_snapshot;
-
-  GdkPixbuf *snapshot;
-  char *path;
+  char *url;
 } SnapshotAsyncData;
 
 static SnapshotAsyncData *
-snapshot_async_data_new (WebKitWebView *web_view,
-                         time_t         mtime)
+snapshot_async_data_new (EphySnapshotService *service,
+                         GdkPixbuf           *snapshot,
+                         WebKitWebView       *web_view,
+                         time_t               mtime,
+                         const char          *url)
 {
   SnapshotAsyncData *data;
 
   data = g_slice_new0 (SnapshotAsyncData);
+  data->service = g_object_ref (service);
+  data->snapshot = snapshot ? g_object_ref (snapshot) : NULL;
   data->web_view = web_view;
   data->mtime = mtime;
+  data->url = g_strdup (url);
 
-  g_object_add_weak_pointer (G_OBJECT (web_view), (gpointer *)&data->web_view);
+  if (web_view)
+    g_object_add_weak_pointer (G_OBJECT (web_view), (gpointer *)&data->web_view);
 
   return data;
 }
 
 static SnapshotAsyncData *
-snapshot_async_data_new_for_snapshot (WebKitWebView *web_view,
-                                      time_t         mtime)
+snapshot_async_data_copy (SnapshotAsyncData *data)
 {
-  SnapshotAsyncData *data = snapshot_async_data_new (web_view, mtime);
-
-  data->for_snapshot = TRUE;
-
-  return data;
+  SnapshotAsyncData *copy = snapshot_async_data_new (data->service,
+                                                     data->snapshot,
+                                                     data->web_view,
+                                                     data->mtime,
+                                                     data->url);
+  return copy;
 }
 
 static void
 snapshot_async_data_free (SnapshotAsyncData *data)
 {
+  g_clear_object (&data->service);
+  g_clear_object (&data->snapshot);
+
   if (data->web_view)
     g_object_remove_weak_pointer (G_OBJECT (data->web_view), (gpointer *)&data->web_view);
-  g_clear_object (&data->snapshot);
-  g_free (data->path);
+
+  g_free (data->url);
 
   g_slice_free (SnapshotAsyncData, data);
 }
 
+typedef struct {
+  GHashTable *cache;
+  char *url;
+  SnapshotPathCachedData *data;
+} CacheData;
+
+static gboolean
+idle_cache_snapshot_path (gpointer user_data)
+{
+  CacheData *data = (CacheData *)user_data;
+  g_hash_table_insert (data->cache, data->url, data->data);
+  g_hash_table_unref (data->cache);
+  g_free (data);
+
+  return G_SOURCE_REMOVE;
+}
+
+static void
+cache_snapshot_data_in_idle (EphySnapshotService  *service,
+                             const char           *url,
+                             const char           *path,
+                             EphySnapshotFreshness freshness)
+{
+  CacheData *data;
+  data = g_new (CacheData, 1);
+  data->cache = g_hash_table_ref (service->cache);
+  data->url = g_strdup (url);
+  data->data = g_new (SnapshotPathCachedData, 1);
+  data->data->path = g_strdup (path);
+  data->data->freshness = freshness;
+  g_idle_add (idle_cache_snapshot_path, data);
+}
+
+static gboolean
+idle_emit_snapshot_saved (gpointer user_data)
+{
+  SnapshotAsyncData *data = (SnapshotAsyncData *)user_data;
+
+  g_signal_emit (data->service, signals[SNAPSHOT_SAVED], 0, data->url, data->mtime);
+
+  snapshot_async_data_free (data);
+  return G_SOURCE_REMOVE;
+}
+
+static void
+save_snapshot_thread (GTask               *task,
+                      EphySnapshotService *service,
+                      SnapshotAsyncData   *data,
+                      GCancellable        *cancellable)
+{
+  char *path;
+
+  gnome_desktop_thumbnail_factory_save_thumbnail (service->factory,
+                                                  data->snapshot,
+                                                  data->url,
+                                                  data->mtime);
+  g_idle_add (idle_emit_snapshot_saved, snapshot_async_data_copy (data));
+
+  path = gnome_desktop_thumbnail_path_for_uri (data->url, GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE);
+  cache_snapshot_data_in_idle (service, data->url, path, SNAPSHOT_FRESH);
+
+  g_task_return_pointer (task, path, g_free);
+}
+
+static void
+ephy_snapshot_service_save_snapshot_async (EphySnapshotService *service,
+                                           GdkPixbuf           *snapshot,
+                                           const char          *url,
+                                           time_t               mtime,
+                                           GCancellable        *cancellable,
+                                           GAsyncReadyCallback  callback,
+                                           gpointer             user_data)
+{
+  GTask *task;
+
+  g_return_if_fail (EPHY_IS_SNAPSHOT_SERVICE (service));
+  g_return_if_fail (GDK_IS_PIXBUF (snapshot));
+  g_return_if_fail (url != NULL);
+
+  task = g_task_new (service, cancellable, callback, user_data);
+  g_task_set_priority (task, G_PRIORITY_LOW);
+  g_task_set_task_data (task,
+                        snapshot_async_data_new (service, snapshot, NULL, mtime, url),
+                        (GDestroyNotify)snapshot_async_data_free);
+  g_task_run_in_thread (task, (GTaskThreadFunc)save_snapshot_thread);
+  g_object_unref (task);
+}
+
+static char *
+ephy_snapshot_service_save_snapshot_finish (EphySnapshotService *service,
+                                            GAsyncResult        *result,
+                                            GError             **error)
+{
+  g_return_val_if_fail (g_task_is_valid (result, service), NULL);
+
+  return g_task_propagate_pointer (G_TASK (result), error);
+}
+
+
 static void
 snapshot_saved (EphySnapshotService *service,
                 GAsyncResult        *result,
                 GTask               *task)
 {
-  SnapshotAsyncData *data = g_task_get_task_data (task);
   char *path;
 
   path = ephy_snapshot_service_save_snapshot_finish (service, result, NULL);
-  if (data->for_snapshot) {
-    data->path = path;
-    g_task_return_pointer (task, g_object_ref (data->snapshot), g_object_unref);
-  } else {
-    g_task_return_pointer (task, path, g_free);
-  }
+  g_task_return_pointer (task, path, g_free);
   g_object_unref (task);
 }
 
@@ -362,8 +489,8 @@ ephy_snapshot_service_get_default (void)
 }
 
 const char *
-ephy_snapshot_service_lookup_snapshot_path (EphySnapshotService *service,
-                                            const char          *url)
+ephy_snapshot_service_lookup_cached_snapshot_path (EphySnapshotService *service,
+                                                   const char          *url)
 {
   SnapshotPathCachedData *data;
 
@@ -385,365 +512,11 @@ ephy_snapshot_service_lookup_snapshot_freshness (EphySnapshotService *service,
   return data == NULL ? SNAPSHOT_STALE : data->freshness;
 }
 
-typedef struct {
-  char *url;
-  time_t mtime;
-
-  char *path;
-} SnapshotForURLAsyncData;
-
-static SnapshotForURLAsyncData *
-snapshot_for_url_async_data_new (const char *url,
-                                 time_t      mtime)
-{
-  SnapshotForURLAsyncData *data;
-
-  data = g_slice_new0 (SnapshotForURLAsyncData);
-  data->url = g_strdup (url);
-  data->mtime = mtime;
-
-  return data;
-}
-
 static void
-snapshot_for_url_async_data_free (SnapshotForURLAsyncData *data)
-{
-  g_free (data->url);
-  g_free (data->path);
-
-  g_slice_free (SnapshotForURLAsyncData, data);
-}
-
-typedef struct {
-  GHashTable *cache;
-  char *url;
-  SnapshotPathCachedData *data;
-} CacheData;
-
-static gboolean
-idle_cache_snapshot_path (gpointer user_data)
-{
-  CacheData *data = (CacheData *)user_data;
-  g_hash_table_insert (data->cache, data->url, data->data);
-  g_hash_table_unref (data->cache);
-  g_free (data);
-
-  return G_SOURCE_REMOVE;
-}
-
-static void
-cache_snapshot_data_in_idle (EphySnapshotService  *service,
-                             const char           *url,
-                             const char           *path,
-                             EphySnapshotFreshness freshness)
-{
-  CacheData *data;
-  data = g_new (CacheData, 1);
-  data->cache = g_hash_table_ref (service->cache);
-  data->url = g_strdup (url);
-  data->data = g_new (SnapshotPathCachedData, 1);
-  data->data->path = g_strdup (path);
-  data->data->freshness = freshness;
-  g_idle_add (idle_cache_snapshot_path, data);
-}
-
-static void
-get_snapshot_for_url_thread (GTask                   *task,
-                             EphySnapshotService     *service,
-                             SnapshotForURLAsyncData *data,
-                             GCancellable            *cancellable)
-{
-  GdkPixbuf *snapshot;
-  GError *error = NULL;
-
-  data->path = gnome_desktop_thumbnail_factory_lookup (service->factory, data->url, data->mtime);
-  if (data->path == NULL) {
-    g_task_return_new_error (task,
-                             EPHY_SNAPSHOT_SERVICE_ERROR,
-                             EPHY_SNAPSHOT_SERVICE_ERROR_NOT_FOUND,
-                             "Snapshot for url \"%s\" not found in cache", data->url);
-    return;
-  }
-
-  cache_snapshot_data_in_idle (service, data->url, data->path, SNAPSHOT_STALE);
-
-  snapshot = gdk_pixbuf_new_from_file (data->path, &error);
-  if (snapshot == NULL) {
-    g_task_return_new_error (task,
-                             EPHY_SNAPSHOT_SERVICE_ERROR,
-                             EPHY_SNAPSHOT_SERVICE_ERROR_INVALID,
-                             "Error creating pixbuf for snapshot file \"%s\": %s",
-                             data->path, error->message);
-    g_error_free (error);
-  }
-
-  g_task_return_pointer (task, snapshot, g_object_unref);
-}
-
-/**
- * ephy_snapshot_service_get_snapshot_for_url:
- * @service: a #EphySnapshotService
- * @url: the URL for which a snapshot is needed
- * @mtime: @the last
- * @callback: a #EphySnapshotServiceCallback
- * @user_data: user data to pass to @callback
- *
- * Schedules a query for a snapshot of @url. If there is an up-to-date
- * snapshot in the cache, this will be retrieved.
- *
- **/
-void
-ephy_snapshot_service_get_snapshot_for_url_async (EphySnapshotService *service,
-                                                  const char          *url,
-                                                  const time_t         mtime,
-                                                  GCancellable        *cancellable,
-                                                  GAsyncReadyCallback  callback,
-                                                  gpointer             user_data)
-{
-  GTask *task;
-
-  g_return_if_fail (EPHY_IS_SNAPSHOT_SERVICE (service));
-  g_return_if_fail (url != NULL);
-
-  task = g_task_new (service, cancellable, callback, user_data);
-  g_task_set_priority (task, G_PRIORITY_LOW);
-  g_task_set_task_data (task,
-                        snapshot_for_url_async_data_new (url, mtime),
-                        (GDestroyNotify)snapshot_for_url_async_data_free);
-  g_task_run_in_thread (task, (GTaskThreadFunc)get_snapshot_for_url_thread);
-  g_object_unref (task);
-}
-
-/**
- * ephy_snapshot_service_get_snapshot_for_url_finish:
- * @service: a #EphySnapshotService
- * @result: a #GAsyncResult
- * @error: a location to store a #GError or %NULL
- *
- * Finishes the retrieval of a snapshot. Call from the
- * #GAsyncReadyCallback passed to
- * ephy_snapshot_service_get_snapshot_for_url_async().
- *
- * Returns: (transfer full): the snapshot.
- **/
-GdkPixbuf *
-ephy_snapshot_service_get_snapshot_for_url_finish (EphySnapshotService *service,
-                                                   GAsyncResult        *result,
-                                                   gchar              **path,
-                                                   GError             **error)
-{
-  GTask *task = G_TASK (result);
-  GdkPixbuf *snapshot;
-
-  g_return_val_if_fail (g_task_is_valid (result, service), NULL);
-
-  snapshot = g_task_propagate_pointer (task, error);
-  if (!snapshot)
-    return NULL;
-
-  if (path) {
-    SnapshotForURLAsyncData *data;
-
-    data = g_task_get_task_data (task);
-    *path = data->path;
-    data->path = NULL;
-  }
-
-  return snapshot;
-}
-
-static void
-got_snapshot_for_url (EphySnapshotService *service,
-                      GAsyncResult        *result,
-                      GTask               *task)
-{
-  GdkPixbuf *snapshot;
-  SnapshotAsyncData *data;
-
-  data = g_task_get_task_data (task);
-  snapshot = ephy_snapshot_service_get_snapshot_for_url_finish (service, result, &data->path, NULL);
-  if (snapshot) {
-    g_task_return_pointer (task, snapshot, g_object_unref);
-    g_object_unref (task);
-    return;
-  }
-
-  ephy_snapshot_service_take_from_webview (task);
-}
-
-/**
- * ephy_snapshot_service_get_snapshot_async:
- * @service: a #EphySnapshotService
- * @web_view: the #WebKitWebView for which a snapshot is needed
- * @mtime: @the last
- * @callback: a #EphySnapshotServiceCallback
- * @user_data: user data to pass to @callback
- *
- * Schedules a query for a snapshot of @url. If there is an up-to-date
- * snapshot in the cache, this will be retrieved. Otherwise, this
- * the snapshot will be taken, cached, and retrieved.
- *
- **/
-void
-ephy_snapshot_service_get_snapshot_async (EphySnapshotService *service,
-                                          WebKitWebView       *web_view,
-                                          const time_t         mtime,
-                                          GCancellable        *cancellable,
-                                          GAsyncReadyCallback  callback,
-                                          gpointer             user_data)
-{
-  GTask *task;
-  const char *uri;
-
-  g_return_if_fail (EPHY_IS_SNAPSHOT_SERVICE (service));
-  g_return_if_fail (WEBKIT_IS_WEB_VIEW (web_view));
-
-  task = g_task_new (service, cancellable, callback, user_data);
-  g_task_set_task_data (task,
-                        snapshot_async_data_new_for_snapshot (web_view, mtime),
-                        (GDestroyNotify)snapshot_async_data_free);
-
-  /* Try to get the snapshot from the cache first if we have a URL, but only if
-   * the snapshot path is in memory cache; this is an indication that the
-   * snapshot is fresh. */
-  uri = webkit_web_view_get_uri (web_view);
-  if (uri && ephy_snapshot_service_lookup_snapshot_freshness (service, uri) == SNAPSHOT_FRESH) {
-    ephy_snapshot_service_get_snapshot_for_url_async (service,
-                                                      uri, mtime, cancellable,
-                                                      (GAsyncReadyCallback)got_snapshot_for_url,
-                                                      task);
-  } else {
-    g_idle_add ((GSourceFunc)ephy_snapshot_service_take_from_webview, task);
-  }
-}
-
-/**
- * ephy_snapshot_service_get_snapshot_finish:
- * @service: a #EphySnapshotService
- * @result: a #GAsyncResult
- * @error: a location to store a #GError or %NULL
- *
- * Finishes the retrieval of a snapshot. Call from the
- * #GAsyncReadyCallback passed to
- * ephy_snapshot_service_get_snapshot_async().
- *
- * Returns: (transfer full): the snapshot.
- **/
-GdkPixbuf *
-ephy_snapshot_service_get_snapshot_finish (EphySnapshotService *service,
-                                           GAsyncResult        *result,
-                                           gchar              **path,
-                                           GError             **error)
-{
-  GTask *task = G_TASK (result);
-  GdkPixbuf *snapshot;
-
-  g_return_val_if_fail (g_task_is_valid (result, service), NULL);
-
-  snapshot = g_task_propagate_pointer (task, error);
-  if (!snapshot)
-    return NULL;
-
-  if (path) {
-    SnapshotAsyncData *data;
-
-    data = g_task_get_task_data (task);
-    *path = data->path;
-    data->path = NULL;
-  }
-
-  return snapshot;
-}
-
-typedef struct {
-  GdkPixbuf *snapshot;
-  char *url;
-  time_t mtime;
-} SaveSnapshotAsyncData;
-
-static SaveSnapshotAsyncData *
-save_snapshot_async_data_new (GdkPixbuf  *snapshot,
-                              const char *url,
-                              time_t      mtime)
-{
-  SaveSnapshotAsyncData *data;
-
-  data = g_slice_new0 (SaveSnapshotAsyncData);
-  data->snapshot = g_object_ref (snapshot);
-  data->url = g_strdup (url);
-  data->mtime = mtime;
-
-  return data;
-}
-
-static void
-save_snapshot_async_data_free (SaveSnapshotAsyncData *data)
-{
-  g_object_unref (data->snapshot);
-  g_free (data->url);
-
-  g_slice_free (SaveSnapshotAsyncData, data);
-}
-
-static void
-save_snapshot_thread (GTask                 *task,
-                      EphySnapshotService   *service,
-                      SaveSnapshotAsyncData *data,
-                      GCancellable          *cancellable)
-{
-  char *path;
-
-  gnome_desktop_thumbnail_factory_save_thumbnail (service->factory,
-                                                  data->snapshot,
-                                                  data->url,
-                                                  data->mtime);
-
-  path = gnome_desktop_thumbnail_path_for_uri (data->url, GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE);
-
-  cache_snapshot_data_in_idle (service, data->url, path, SNAPSHOT_FRESH);
-
-  g_task_return_pointer (task, path, g_free);
-}
-
-void
-ephy_snapshot_service_save_snapshot_async (EphySnapshotService *service,
-                                           GdkPixbuf           *snapshot,
-                                           const char          *url,
-                                           time_t               mtime,
-                                           GCancellable        *cancellable,
-                                           GAsyncReadyCallback  callback,
-                                           gpointer             user_data)
-{
-  GTask *task;
-
-  g_return_if_fail (EPHY_IS_SNAPSHOT_SERVICE (service));
-  g_return_if_fail (GDK_IS_PIXBUF (snapshot));
-  g_return_if_fail (url != NULL);
-
-  task = g_task_new (service, cancellable, callback, user_data);
-  g_task_set_priority (task, G_PRIORITY_LOW);
-  g_task_set_task_data (task,
-                        save_snapshot_async_data_new (snapshot, url, mtime),
-                        (GDestroyNotify)save_snapshot_async_data_free);
-  g_task_run_in_thread (task, (GTaskThreadFunc)save_snapshot_thread);
-  g_object_unref (task);
-}
-
-char *
-ephy_snapshot_service_save_snapshot_finish (EphySnapshotService *service,
-                                            GAsyncResult        *result,
-                                            GError             **error)
-{
-  g_return_val_if_fail (g_task_is_valid (result, service), NULL);
-
-  return g_task_propagate_pointer (G_TASK (result), error);
-}
-
-static void
-get_snapshot_path_for_url_thread (GTask                   *task,
-                                  EphySnapshotService     *service,
-                                  SnapshotForURLAsyncData *data,
-                                  GCancellable            *cancellable)
+get_snapshot_path_for_url_thread (GTask               *task,
+                                  EphySnapshotService *service,
+                                  SnapshotAsyncData   *data,
+                                  GCancellable        *cancellable)
 {
   char *path;
 
@@ -752,7 +525,7 @@ get_snapshot_path_for_url_thread (GTask                   *task,
     g_task_return_new_error (task,
                              EPHY_SNAPSHOT_SERVICE_ERROR,
                              EPHY_SNAPSHOT_SERVICE_ERROR_NOT_FOUND,
-                             "Snapshot for url \"%s\" not found in cache", data->url);
+                             "Snapshot for url \"%s\" not found in disk cache", data->url);
     return;
   }
 
@@ -764,7 +537,7 @@ get_snapshot_path_for_url_thread (GTask                   *task,
 void
 ephy_snapshot_service_get_snapshot_path_for_url_async (EphySnapshotService *service,
                                                        const char          *url,
-                                                       const time_t         mtime,
+                                                       time_t               mtime,
                                                        GCancellable        *cancellable,
                                                        GAsyncReadyCallback  callback,
                                                        gpointer             user_data)
@@ -777,7 +550,7 @@ ephy_snapshot_service_get_snapshot_path_for_url_async (EphySnapshotService *serv
 
   task = g_task_new (service, cancellable, callback, user_data);
 
-  path = ephy_snapshot_service_lookup_snapshot_path (service, url);
+  path = ephy_snapshot_service_lookup_cached_snapshot_path (service, url);
   if (path) {
     g_task_return_pointer (task, g_strdup (path), g_free);
     g_object_unref (task);
@@ -786,8 +559,8 @@ ephy_snapshot_service_get_snapshot_path_for_url_async (EphySnapshotService *serv
 
   g_task_set_priority (task, G_PRIORITY_LOW);
   g_task_set_task_data (task,
-                        snapshot_for_url_async_data_new (url, mtime),
-                        (GDestroyNotify)snapshot_for_url_async_data_free);
+                        snapshot_async_data_new (service, NULL, NULL, mtime, url),
+                        (GDestroyNotify)snapshot_async_data_free);
   g_task_run_in_thread (task, (GTaskThreadFunc)get_snapshot_path_for_url_thread);
   g_object_unref (task);
 }
@@ -807,54 +580,59 @@ got_snapshot_path_for_url (EphySnapshotService *service,
                            GAsyncResult        *result,
                            GTask               *task)
 {
+  SnapshotAsyncData *data = g_task_get_task_data (task);
   char *path;
 
   path = ephy_snapshot_service_get_snapshot_path_for_url_finish (service, result, NULL);
   if (path) {
     g_task_return_pointer (task, path, g_free);
     g_object_unref (task);
-  } else {
-    ephy_snapshot_service_take_from_webview (task);
+
+    if (ephy_snapshot_service_lookup_snapshot_freshness (service, data->url) == SNAPSHOT_FRESH)
+      return;
+
+    /* Take a fresh snapshot in the background. */
+    task = g_task_new (service, NULL, NULL, NULL);
+    g_task_set_task_data (task,
+                          snapshot_async_data_copy (data),
+                          (GDestroyNotify)snapshot_async_data_free);
   }
+
+  ephy_snapshot_service_take_from_webview (task);
 }
 
 void
 ephy_snapshot_service_get_snapshot_path_async (EphySnapshotService *service,
                                                WebKitWebView       *web_view,
-                                               const time_t         mtime,
+                                               time_t               mtime,
                                                GCancellable        *cancellable,
                                                GAsyncReadyCallback  callback,
                                                gpointer             user_data)
 {
   GTask *task;
   const char *uri;
+  const char *path;
 
   g_return_if_fail (EPHY_IS_SNAPSHOT_SERVICE (service));
   g_return_if_fail (WEBKIT_IS_WEB_VIEW (web_view));
+  g_return_if_fail (webkit_web_view_get_uri (web_view));
 
   task = g_task_new (service, cancellable, callback, user_data);
 
   uri = webkit_web_view_get_uri (web_view);
-  if (uri && ephy_snapshot_service_lookup_snapshot_freshness (service, uri) == SNAPSHOT_FRESH) {
-    const char *path = ephy_snapshot_service_lookup_snapshot_path (service, uri);
+  path = ephy_snapshot_service_lookup_cached_snapshot_path (service, uri);
 
-    if (path) {
-      g_task_return_pointer (task, g_strdup (path), g_free);
-      g_object_unref (task);
-    } else {
-      g_task_set_task_data (task,
-                            snapshot_async_data_new (web_view, mtime),
-                            (GDestroyNotify)snapshot_async_data_free);
-      ephy_snapshot_service_get_snapshot_path_for_url_async (service,
-                                                             uri, mtime, cancellable,
-                                                             (GAsyncReadyCallback)got_snapshot_path_for_url,
-                                                             task);
-    }
+  if (path) {
+    g_task_return_pointer (task, g_strdup (path), g_free);
+    g_object_unref (task);
   } else {
     g_task_set_task_data (task,
-                          snapshot_async_data_new (web_view, mtime),
+                          snapshot_async_data_new (service, NULL, web_view, mtime, uri),
                           (GDestroyNotify)snapshot_async_data_free);
-    g_idle_add ((GSourceFunc)ephy_snapshot_service_take_from_webview, task);
+    ephy_snapshot_service_get_snapshot_path_for_url_async (service,
+                                                           uri, mtime, cancellable,
+                                                           (GAsyncReadyCallback)got_snapshot_path_for_url,
+                                                           task);
   }
 }
 
